package games;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.Scanner;

public class ChristmasHangman {
    private static final String COLOR_RESET = "\u001B[0m";
    private static final String COLOR_RED = "\u001B[31m";
    private static final String COLOR_GREEN = "\u001B[32m";
    private static final String COLOR_BLUE = "\u001B[34m";

    private List<String> words;
    private int rounds;
    private static final int MAX_WRONG_GUESSES = 10;
    public  ChristmasHangman(String dictionaryPath, int rounds) throws IOException {
        this.words = Files.readAllLines(Paths.get(dictionaryPath));
        this.rounds = rounds;
    }

    public void startGame() {
        Scanner scanner = new Scanner(System.in);
        int currentRound = 1;

        while (currentRound <= rounds) {
            System.out.println("\nKolo " + currentRound + "/" + rounds);
            String word = getRandomWord().toUpperCase();
            playRound(word, scanner);
            currentRound++;
        }

        System.out.println("\nHra skončila. Ďakujeme za hranie!");
    }

    private String getRandomWord() {
        Random random = new Random();
        return words.get(random.nextInt(words.size()));
    }

    private void playRound(String word, Scanner scanner) {
        StringBuilder hiddenWord = initializeHiddenWord(word);
        List<Character> wrongGuesses = new ArrayList<>();
        int wrongGuessCount = 0;

        while (wrongGuessCount < MAX_WRONG_GUESSES) {
            System.out.println("\nTajnička: " + hiddenWord);
            System.out.println("Zlé tipy: " + formatWrongGuesses(wrongGuesses));
            System.out.print("Hádať písmeno: ");
            String guess = scanner.nextLine().toUpperCase();

            if (guess.length() != 1 || !Character.isLetter(guess.charAt(0))) {
                System.out.println("Neplatný vstup! Skús jedno písmeno.");
                continue;
            }

            char guessedChar = guess.charAt(0);
            if (hiddenWord.toString().contains(String.valueOf(guessedChar)) || wrongGuesses.contains(guessedChar)) {
                System.out.println("Písmeno už bolo hádané! Skús iné.");
                continue;
            }

            if (word.contains(String.valueOf(guessedChar))) {
                revealLetter(hiddenWord, word, guessedChar);
                System.out.println(COLOR_GREEN + "Písmeno " + guessedChar + " bolo doplnené." + COLOR_RESET);
                if (!hiddenWord.toString().contains("_")) {
                    System.out.println("\nGratulujem! Uhadol si výraz: " + word);
                    return;
                }
            } else {
                wrongGuesses.add(guessedChar);
                wrongGuessCount++;
                System.out.println(COLOR_RED + "Písmeno " + guessedChar + " nie je v tajničke." + COLOR_RESET);
                System.out.println("Stav: " + "KONIEC HRY".substring(0, wrongGuessCount));
            }
        }

        System.out.println("\nPrehral si! Hľadaný výraz bol: " + word);
    }

    private StringBuilder initializeHiddenWord(String word) {
        StringBuilder hiddenWord = new StringBuilder();
        for (char c : word.toCharArray()) {
            if (Character.isLetter(c)) {
                hiddenWord.append("_ ");
            } else {
                hiddenWord.append(c).append(" ");
            }
        }
        hiddenWord.deleteCharAt(hiddenWord.length() - 1); // Remove last space
        return hiddenWord;
    }

    private void revealLetter(StringBuilder hiddenWord, String word, char guessedChar) {
        for (int i = 0; i < word.length(); i++) {
            if (word.charAt(i) == guessedChar) {
                hiddenWord.setCharAt(i * 2, guessedChar);
            }
        }
    }

    private String formatWrongGuesses(List<Character> wrongGuesses) {
        StringBuilder formatted = new StringBuilder();
        for (char c : wrongGuesses) {
            formatted.append(COLOR_BLUE).append(c).append(COLOR_RESET).append(", ");
        }
        return formatted.length() > 0 ? formatted.substring(0, formatted.length() - 2) : "žiadne";
    }
}
